//-----------------------------------------------------------------------------
//  Mandelbrot Set Generator
//  ECE573 Project, Winter 2008
//  Author:  Jesse Armagost
//-----------------------------------------------------------------------------


`include "mand_defs.v"
`include "vga_defs_1024_768.v"

module cursor ( input                               sys_reset_n,
                input                               sysclk,
                input                               cursor_en_dis,
                input                               mand_pan_up,
                input                               mand_pan_down,
                input                               mand_pan_left,
                input                               mand_pan_right,
                input                               mand_start,
                input                               mand_restart,
                input       [`LB2_PIXELS_H - 1:0]   vga_h_coord,
                input       [`LB2_PIXELS_V - 1:0]   vga_v_coord,
                output reg                          cursor_draw_en,
                output reg  [3:0]                   cursor_palette_index  );

  //regs and wires
  reg [`LB2_PIXELS_H - 1:0] cursor_center_h;
  reg [`LB2_PIXELS_V - 1:0] cursor_center_v;
  wire cursor_blink_en;
  reg [23:0] lfsr;
  reg cursor_en;

  //the cursor is enabled or disabled with keyboard
  always @(negedge sys_reset_n or posedge sysclk) begin
    if (~sys_reset_n)
      cursor_en <= 1'b0;
    else if (sysclk) begin
      if (cursor_en_dis)
        cursor_en <= ~cursor_en;
    end
  end

  //maintain the lfsr that we use to blink the cursor
  always @(negedge sys_reset_n or posedge sysclk) begin
    if (~sys_reset_n)
      lfsr <= 24'hffffff;
    else if (sysclk)
      lfsr <= {lfsr[22:0], lfsr[23] ^ lfsr[22] ^ lfsr[21] ^ lfsr[16]};
  end

  assign cursor_blink_en = (lfsr == 24'hffffff) ? 1'b1 : 1'b0;

  //maintain the cursor color
  always @(negedge sys_reset_n or posedge sysclk) begin
    if (~sys_reset_n)
      cursor_palette_index <= 4'h2;
    else if (sysclk) begin
      if (cursor_blink_en)
        cursor_palette_index <= ~cursor_palette_index;
    end
  end

  //maintain the current position of the cursor on the screen.
  always @(negedge sys_reset_n or posedge sysclk) begin
    if (~sys_reset_n) begin
      cursor_center_h <= `H_PIXELS / 2;
      cursor_center_v <= `V_PIXELS / 2;
    end
    else if (sysclk) begin
      if (mand_start | mand_restart) begin
        cursor_center_h <= `H_PIXELS / 2;
        cursor_center_v <= `V_PIXELS / 2;
      end
      if (mand_pan_up) begin
        cursor_center_v <= cursor_center_v - `PAN_PIXELS;
      end
      if (mand_pan_down) begin
        cursor_center_v <= cursor_center_v + `PAN_PIXELS;
      end
      if (mand_pan_left) begin
        cursor_center_h <= cursor_center_h - `PAN_PIXELS;
      end
      if (mand_pan_right) begin
        cursor_center_h <= cursor_center_h + `PAN_PIXELS;
      end
    end
  end

  //determine if we should be drawing the cursor on the screen right now.
  always @(*) begin
    if (cursor_en &
          (
            (     vga_v_coord == cursor_center_v
              &&  vga_h_coord > (cursor_center_h - `CURSOR_SIZE / 2)
              &&  vga_h_coord < (cursor_center_h + `CURSOR_SIZE / 2) )
             ||
            (     vga_h_coord == cursor_center_h
              &&  vga_v_coord > (cursor_center_v - `CURSOR_SIZE / 2)
              &&  vga_v_coord < (cursor_center_v + `CURSOR_SIZE / 2) )
          )
        )
      cursor_draw_en <= 1'b1;
    else
      cursor_draw_en <= 1'b0;
  end


endmodule



