//-----------------------------------------------------------------------------
//  Mandelbrot Set Generator
//  ECE573 Project, Winter 2008
//  Author:  Jesse Armagost
//-----------------------------------------------------------------------------

//total number of fixed-point math digits, including sign bit.
`define DIGITS 32

//total number of digits after the decimal point.  to convert to fixed point
//  math, multiply by 2^`FRAC_DIGITS.  to convert back, divide by 2^`FRAC_DIGITS
`define FRAC_DIGITS 29

//the width of the front end fifo.  depends on the value of DIGITS.  pixel
//  values are fixed at 10 bits each.  the front end fifo holds two complex
//  digits and two pixel values.
`define FE_FIFO_WIDTH (2 * `DIGITS + 20)

//the back end fifo width is a constant.  the backend fifo holds two pixel
//  values, the iterations count, and in_set
`define BE_FIFO_WIDTH (2 * 10 + 16 + 1)

//number of complex pipes
`define NUM_PIPES 2
`define LB2_NUM_PIPES 1

//screen size in pixels
`define H_PIXELS 11'd1024
`define V_PIXELS 11'd768

//the break-out iteration requirement.  if we iterate this many times, the point
//  is in the set.
`define MAX_ITER 16'd1024

//the complex plane center point upon reset.  (-0.75, 0)
`define INITIAL_COMPLEX_CENTER_RE -`DIGITS'sd402653184
`define INITIAL_COMPLEX_CENTER_IM `DIGITS'sd0

//the initial complex plane pixel delta.  this is how much we move in the
//  complex plane when we move one pixel vertically or horizontally.
`define INITIAL_COMPLEX_PIXEL_DELTA `DIGITS'sd1565874  //  0.0029

//the number of pixels we move when we pan up, down, left or right.
`define PAN_PIXELS 4

//the complex plane zooming factor.  this is how much we zoom in or out.
`define COMPLEX_PLANE_ZOOM_FACTOR `DIGITS'sd2

//the size of the cursor
`define CURSOR_SIZE 16




