//-----------------------------------------------------------------------------
//  Mandelbrot Set Generator
//  ECE573 Project, Winter 2008
//  Author:  Jesse Armagost
//-----------------------------------------------------------------------------

`timescale 1 ps / 1 ps

`include "mand_defs.v"
`include "vga_defs_1024_768.v"

module top
  ( //clocks
    CLOCK_24,
    CLOCK_27,
    CLOCK_50,
    EXT_CLOCK,
    //push buttons
    KEY,                            //Pushbutton[3:0]
    //switches
    SW,                             //Toggle Switch[9:0]
    //seven segment displays
    HEX0,                           //Seven Segment Digit 0
    HEX1,                           //Seven Segment Digit 1
    HEX2,                           //Seven Segment Digit 2
    HEX3,                           //Seven Segment Digit 3
    //leds
    LEDG,                           //LED Green[7:0]
    LEDR,                           //LED Red[9:0]
    //uart
    UART_TXD,                       //UART Transmitter
    UART_RXD,                       //UART Receiver
    //sdram
    DRAM_DQ,                        //SDRAM Data bus 16 Bits
    DRAM_ADDR,                      //SDRAM Address bus 12 Bits
    DRAM_LDQM,                      //SDRAM Low-byte Data Mask
    DRAM_UDQM,                      //SDRAM High-byte Data Mask
    DRAM_WE_N,                      //SDRAM Write Enable
    DRAM_CAS_N,                     //SDRAM Column Address Strobe
    DRAM_RAS_N,                     //SDRAM Row Address Strobe
    DRAM_CS_N,                      //SDRAM Chip Select
    DRAM_BA_0,                      //SDRAM Bank Address 0
    DRAM_BA_1,                      //SDRAM Bank Address 0
    DRAM_CLK,                       //SDRAM Clock
    DRAM_CKE,                       //SDRAM Clock Enable
    //flash
    FL_DQ,                          //FLASH Data bus 8 Bits
    FL_ADDR,                        //FLASH Address bus 22 Bits
    FL_WE_N,                        //FLASH Write Enable
    FL_RST_N,                       //FLASH Reset
    FL_OE_N,                        //FLASH Output Enable
    FL_CE_N,                        //FLASH Chip Enable
    //sram
    SRAM_DQ,                        //SRAM Data bus 16 Bits
    SRAM_ADDR,                      //SRAM Address bus 18 Bits
    SRAM_UB_N,                      //SRAM High-byte Data Mask
    SRAM_LB_N,                      //SRAM Low-byte Data Mask
    SRAM_WE_N,                      //SRAM Write Enable
    SRAM_CE_N,                      //SRAM Chip Enable
    SRAM_OE_N,                      //SRAM Output Enable
    //sd card socket
    SD_DAT,                         //SD Card Data
    SD_DAT3,                        //SD Card Data 3
    SD_CMD,                         //SD Card Command Signal
    SD_CLK,                         //SD Card Clock
    //usb jtag link
    TDI,                            //CPLD -> FPGA (data in)
    TCK,                            //CPLD -> FPGA (clk)
    TCS,                            //CPLD -> FPGA (CS)
    TDO,                            //FPGA -> CPLD (data out)
    //i2c
    I2C_SDAT,                       //I2C Data
    I2C_SCLK,                       //I2C Clock
    //ps2
    PS2_DAT,                        //PS2 Data
    PS2_CLK,                        //PS2 Clock
    //vga
    VGA_HS,                         //VGA H_SYNC
    VGA_VS,                         //VGA V_SYNC
    VGA_R,                          //VGA Red[3:0]
    VGA_G,                          //VGA Green[3:0]
    VGA_B,                          //VGA Blue[3:0]
    //audio codec
    AUD_ADCLRCK,                    //Audio CODEC ADC LR Clock
    AUD_ADCDAT,                     //Audio CODEC ADC Data
    AUD_DACLRCK,                    //Audio CODEC DAC LR Clock
    AUD_DACDAT,                     //Audio CODEC DAC Data
    AUD_BCLK,                       //Audio CODEC Bit-Stream Clock
    AUD_XCK,                        //Audio CODEC Chip Clock
    //gpio
    GPIO_0,                         //GPIO Connection 0
    GPIO_1                          //GPIO Connection 1
  );

  input   [1:0] CLOCK_24;       //24 MHz
  input   [1:0] CLOCK_27;       //27 MHz
  input   CLOCK_50;             //50 MHz
  input   EXT_CLOCK;            //External Clock
  input   [3:0] KEY;            //Pushbutton[3:0]
  input   [9:0] SW;             //Toggle Switch[9:0]
  output  [6:0] HEX0;           //Seven Segment Digit 0
  output  [6:0] HEX1;           //Seven Segment Digit 1
  output  [6:0] HEX2;           //Seven Segment Digit 2
  output  [6:0] HEX3;           //Seven Segment Digit 3
  output  [7:0] LEDG;           //LED Green[7:0]
  output  [9:0] LEDR;           //LED Red[9:0]
  output  UART_TXD;             //UART Transmitter
  input   UART_RXD;             //UART Receiver
  inout   [15:0] DRAM_DQ;       //SDRAM Data bus 16 Bits
  output  [11:0] DRAM_ADDR;     //SDRAM Address bus 12 Bits
  output  DRAM_LDQM;            //SDRAM Low-byte Data Mask
  output  DRAM_UDQM;            //SDRAM High-byte Data Mask
  output  DRAM_WE_N;            //SDRAM Write Enable
  output  DRAM_CAS_N;           //SDRAM Column Address Strobe
  output  DRAM_RAS_N;           //SDRAM Row Address Strobe
  output  DRAM_CS_N;            //SDRAM Chip Select
  output  DRAM_BA_0;            //SDRAM Bank Address 0
  output  DRAM_BA_1;            //SDRAM Bank Address 0
  output  DRAM_CLK;             //SDRAM Clock
  output  DRAM_CKE;             //SDRAM Clock Enable
  inout   [7:0] FL_DQ;          //FLASH Data bus 8 Bits
  output  [21:0] FL_ADDR;       //FLASH Address bus 22 Bits
  output  FL_WE_N;              //FLASH Write Enable
  output  FL_RST_N;             //FLASH Reset
  output  FL_OE_N;              //FLASH Output Enable
  output  FL_CE_N;              //FLASH Chip Enable
  inout   [15:0] SRAM_DQ;       //SRAM Data bus 16 Bits
  output  [17:0] SRAM_ADDR;     //SRAM Address bus 18 Bits
  output  SRAM_UB_N;            //SRAM High-byte Data Mask
  output  SRAM_LB_N;            //SRAM Low-byte Data Mask
  output  SRAM_WE_N;            //SRAM Write Enable
  output  SRAM_CE_N;            //SRAM Chip Enable
  output  SRAM_OE_N;            //SRAM Output Enable
  inout   SD_DAT;               //SD Card Data
  inout   SD_DAT3;              //SD Card Data 3
  inout   SD_CMD;               //SD Card Command Signal
  output  SD_CLK;               //SD Card Clock
  inout   I2C_SDAT;             //I2C Data
  output  I2C_SCLK;             //I2C Clock
  input   PS2_DAT;              //PS2 Data
  input   PS2_CLK;              //PS2 Clock
  input   TDI;                  //CPLD -> FPGA (data in)
  input   TCK;                  //CPLD -> FPGA (clk)
  input   TCS;                  //CPLD -> FPGA (CS)
  output  TDO;                  //FPGA -> CPLD (data out)
  output  VGA_HS;               //VGA H_SYNC
  output  VGA_VS;               //VGA V_SYNC
  output  [3:0] VGA_R;          //VGA Red[3:0]
  output  [3:0] VGA_G;          //VGA Green[3:0]
  output  [3:0] VGA_B;          //VGA Blue[3:0]
  output  AUD_ADCLRCK;          //Audio CODEC ADC LR Clock
  input   AUD_ADCDAT;           //Audio CODEC ADC Data
  output  AUD_DACLRCK;          //Audio CODEC DAC LR Clock
  output  AUD_DACDAT;           //Audio CODEC DAC Data
  inout   AUD_BCLK;             //Audio CODEC Bit-Stream Clock
  output  AUD_XCK;              //Audio CODEC Chip Clock
  inout   [35:0] GPIO_0;        //GPIO Connection 0
  inout   [35:0] GPIO_1;        //GPIO Connection 1

  //regs and wires
  wire vgaclk;
  wire pipeclk;
  wire sysclk;
  wire pll_reset;
  wire pipe_pll_locked;
  wire vga_pll_locked;
  wire plls_locked;
  wire sys_reset_n;
  wire vga_reset_n;
  wire pipe_reset_n;
  wire [9:0] mand_current_line;
  wire mand_rendering;
  wire mand_start;
  wire mand_restart;
  wire mand_left;
  wire mand_right;
  wire mand_up;
  wire mand_down;
  wire mand_in;
  wire mand_out;
  wire [`NUM_PIPES - 1:0] mand_cp_full;
  wire [`NUM_PIPES - 1:0] mand_be_fifo_wrfull;
  wire vga_color_cycle_en;
  wire [`FE_FIFO_WIDTH - 1:0] mand_fe_fifo_wrdata;
  wire mand_fe_fifo_wrreq;
  wire mand_fe_fifo_wrfull;
  wire [`NUM_PIPES - 1:0] mand_be_fifo_rdreq;
  wire [`NUM_PIPES - 1:0] mand_be_fifo_rdempty;
  wire [`NUM_PIPES * `BE_FIFO_WIDTH - 1:0] mand_be_fifo_rddata;
  wire [17:0] mand_sram_addr;
  wire mand_sram_oe_n;
  wire mand_sram_we_n;
  wire [15:0] mand_sram_wr_data;
  wire [17:0] vga_sram_addr;
  wire [3:0] vga_r_in;
  wire [3:0] vga_g_in;
  wire [3:0] vga_b_in;
  wire [9:0] vga_h_coord;
  wire [9:0] vga_v_coord;
  wire vga_sram_wr_start_en;
  wire mand_sram_wr_start_en;
  wire mand_sram_wr_in_progress;
  wire uart_tx_en;
  wire uart_rx_en;
  wire [7:0] uart_rx_data_reg;
  wire frame_done;
  wire uart_echo;
  reg [15:0] sram_dq_1q;
  wire uart_rx_rdy;
  reg [1:0] vga_nibble_select_1q;
  reg [1:0] vga_nibble_select_2q;
  wire [3:0] cursor_palette_index;
  wire cursor_en_dis;
  wire cursor_draw_en;
  reg [3:0] vga_nibble_mand;
  wire [3:0] vga_nibble;


  //create the shared pll reset signal
  assign pll_reset = ~KEY[3];

  //instantiate the complex pipe pll
  pipe_pll i_pipe_pll ( .areset (pll_reset),
                        .inclk0 (CLOCK_24[0]),
                        .c0 (pipeclk),
                        .locked (pipe_pll_locked)  );

  //instantiate the vga pll
  vga_pll i_vga_pll ( .areset (pll_reset),
                      .inclk0 (CLOCK_50),
                      .c0 (vgaclk),
                      .locked (vga_pll_locked)  );

  //create a combined pll locked signal
  assign plls_locked = (pipe_pll_locked & vga_pll_locked);

  //instantiate the reset filter for the pipeclk domain
  reset_filter pipe_reset_filter (  .clk (pipeclk),
                                    .dirty_reset_n (plls_locked),
                                    .clean_reset_n (pipe_reset_n)  );

  //instantiate the reset filter for the vgaclk domain
  reset_filter vga_reset_filter ( .clk (vgaclk),
                                  .dirty_reset_n (plls_locked),
                                  .clean_reset_n (vga_reset_n)  );

  //instantiate the reset filter for the sysclk domain
  reset_filter sys_reset_filter ( .clk (sysclk),
                                  .dirty_reset_n (plls_locked),
                                  .clean_reset_n (sys_reset_n)  );

  //instantiate the mand module
  mand i_mand ( .sys_reset_n (sys_reset_n),
                .sysclk (sysclk),
                .pan_left (mand_left),
                .pan_right (mand_right),
                .pan_up (mand_up),
                .pan_down (mand_down),
                .zoom_in (mand_in),
                .zoom_out (mand_out),
                .start (mand_start),
                .restart (mand_restart),
                .current_line (mand_current_line),
                .rendering (mand_rendering),
                .frame_done (frame_done),
                .sram_wr_start_en (mand_sram_wr_start_en),
                .sram_wr_in_progress (mand_sram_wr_in_progress),
                .zoom_in_en_led (LEDR[0]),
                .uart_rx_en (uart_rx_en),
                .fe_fifo_wrreq (mand_fe_fifo_wrreq),
                .fe_fifo_wrdata (mand_fe_fifo_wrdata),
                .fe_fifo_wrfull (mand_fe_fifo_wrfull),
                .be_fifo_rdreq (mand_be_fifo_rdreq),
                .be_fifo_rddata (mand_be_fifo_rddata),
                .be_fifo_rdempty (mand_be_fifo_rdempty),
                .sram_we_n (mand_sram_we_n),
                .sram_addr (mand_sram_addr),
                .sram_wr_data (mand_sram_wr_data),
                .sram_oe_n (mand_sram_oe_n),
                .sram_rd_data (SRAM_DQ)                   );

  //switch 9 enables the mand module to write to the sram at any time.  the
  //  screen will be gibberish while writing, but then be fine when it's done.
  //  this removes the sram write bottleneck.
  assign mand_sram_wr_start_en = SW[9] | vga_sram_wr_start_en;

  //instantiate the engine
  engine i_engine ( .pipe_reset_n (pipe_reset_n),
                    .pipeclk (pipeclk),
                    .sysclk (sysclk),
                    .fe_fifo_wrdata (mand_fe_fifo_wrdata),
                    .fe_fifo_wrreq (mand_fe_fifo_wrreq),
                    .fe_fifo_wrfull (mand_fe_fifo_wrfull),
                    .cp_full (mand_cp_full),
                    .be_fifo_wrfull (mand_be_fifo_wrfull),
                    .be_fifo_rdreq (mand_be_fifo_rdreq),
                    .be_fifo_rddata (mand_be_fifo_rddata),
                    .be_fifo_rdempty (mand_be_fifo_rdempty) );

  assign LEDG[4] = mand_fe_fifo_wrfull;
  assign LEDG[3:2] = mand_cp_full;
  assign LEDG[1:0] = mand_be_fifo_wrfull;

  //instantiate the sram io module
  sram_io i_sram_io ( .sys_reset_n (sys_reset_n),
                      .sysclk (sysclk),
                      .vga_reset_n (vga_reset_n),
                      .vgaclk (vgaclk),
                      .mand_sram_wr_in_progress (mand_sram_wr_in_progress),
                      .mand_sram_addr (mand_sram_addr),
                      .mand_sram_oe_n (mand_sram_oe_n),
                      .mand_sram_we_n (mand_sram_we_n),
                      .mand_sram_wr_data (mand_sram_wr_data),
                      .vga_sram_addr (vga_sram_addr),
                      .sram_addr (SRAM_ADDR),
                      .sram_oe_n (SRAM_OE_N),
                      .sram_we_n (SRAM_WE_N),
                      .sram_data (SRAM_DQ),
                      .sram_ce_n (SRAM_CE_N),
                      .sram_lb_n (SRAM_LB_N),
                      .sram_ub_n (SRAM_UB_N)                                );

  //form the vga sram address using the current vga horizontal and vertical
  //  coordinates.  at each sram address we store four nibbles, corresponding
  //  to four pixels.
  assign vga_sram_addr = {vga_v_coord, vga_h_coord[9:2]};

  //register the incoming sram data to improve timing.
  always @(negedge vga_reset_n or posedge vgaclk) begin
    if (~vga_reset_n)
      sram_dq_1q <= 0;
    else if (vgaclk)
      sram_dq_1q <= SRAM_DQ;
  end

  assign sysclk = CLOCK_50;

  //create a registered version of the lowest two bits of vga_h_coord to
  //  determine which nibble to grab from the sram read data.  this must be
  //  delayed the same amount as the total sram data latency, so we register it
  //  twice.
  always @(negedge vga_reset_n or posedge vgaclk) begin
    if (~vga_reset_n) begin
      vga_nibble_select_1q <= 0;
      vga_nibble_select_2q <= 0;
    end
    else if (vgaclk) begin
      vga_nibble_select_1q <= vga_h_coord[1:0];
      vga_nibble_select_2q <= vga_nibble_select_1q;
    end
  end

  //pick the correct nibble out of the sram read data for determining vga
  //  pixel data for the mandelbrot set.
  always @(*) begin
    case (vga_nibble_select_2q)
      2'b00 :   vga_nibble_mand <= sram_dq_1q[3:0];
      2'b01 :   vga_nibble_mand <= sram_dq_1q[7:4];
      2'b10 :   vga_nibble_mand <= sram_dq_1q[11:8];
      default : vga_nibble_mand <= sram_dq_1q[15:12];
    endcase
  end

  //instantiate vga controller
  vga_controller i_vga_controller ( .reset_n (vga_reset_n),
                                    .vgaclk (vgaclk),
                                    .r_in (vga_r_in),
                                    .g_in (vga_g_in),
                                    .b_in (vga_b_in),
                                    .h_coord (vga_h_coord),
                                    .v_coord (vga_v_coord),
                                    .r_out (VGA_R),
                                    .g_out (VGA_G),
                                    .b_out (VGA_B),
                                    .h_sync (VGA_HS),
                                    .v_sync (VGA_VS),
                                    .sram_wr_start_en (vga_sram_wr_start_en) );

  //feed the correct data to the vga palette module to produce either the
  //  mandelbrot set or the cursor, if it's enabled.
  assign vga_nibble = (cursor_draw_en & ~mand_rendering)
                                                      ?  cursor_palette_index
                                                      :  vga_nibble_mand;

  //instantiate the vga color palette rom
  vga_palette i_vga_palette ( .vga_reset_n (vga_reset_n),
                              .vgaclk (vgaclk),
                              .color_cycle_en (vga_color_cycle_en),
                              .vga_vsync (VGA_VS),
                              .palette_select (SW[1:0]),
                              .index (vga_nibble),
                              .rgb_data ({vga_r_in, vga_g_in, vga_b_in} )   );

  //instantiate the cursor module
  cursor i_cursor ( .sys_reset_n (sys_reset_n),
                    .sysclk (sysclk),
                    .cursor_en_dis (cursor_en_dis),
                    .mand_pan_up (mand_up),
                    .mand_pan_down (mand_down),
                    .mand_pan_left (mand_left),
                    .mand_pan_right (mand_right),
                    .mand_start (mand_start),
                    .mand_restart (mand_restart),
                    .vga_h_coord (vga_h_coord),
                    .vga_v_coord (vga_v_coord),
                    .cursor_draw_en (cursor_draw_en),
                    .cursor_palette_index (cursor_palette_index)  );

  //instantiate the seven segment display driver bank
  ssd_bank i_ssd_bank ( .hex_value ({6'h00, mand_current_line}),
                        .digit_3 (HEX3),
                        .digit_2 (HEX2),
                        .digit_1 (HEX1),
                        .digit_0 (HEX0)                           );

  //instantiate the uart rx/tx module
  uart i_uart ( .reset_n (sys_reset_n),
                .clk (sysclk),
                .rx_en (uart_rx_en),
                .echo (uart_echo),
                .uart_rx (UART_RXD),
                .uart_tx (UART_TXD),
                .rx_data_reg (uart_rx_data_reg),
                .rx_rdy (uart_rx_rdy)             );

  //instantiate the uart decode module
  uart_decode i_uart_decode ( .reset_n (sys_reset_n),
                              .clk (sysclk),
                              .rx_rdy (uart_rx_rdy),
                              .rx_data_reg (uart_rx_data_reg),
                              .frame_done (frame_done),
                              .echo (uart_echo),
                              .start (mand_start),
                              .restart (mand_restart),
                              .left (mand_left),
                              .right (mand_right),
                              .up (mand_up),
                              .down (mand_down),
                              .in (mand_in),
                              .out (mand_out),
                              .anim (vga_color_cycle_en),
                              .cursor_en_dis (cursor_en_dis)    );


  //disable unused peripherals
  assign LEDR[9:1] = 0;
  assign LEDG[7:5] = 0;
  assign FL_CE_N = 1'b1;
  assign FL_OE_N = 1'b1;
  assign FL_RST_N = 1'b0;
  assign FL_WE_N = 1'b1;
  assign FL_DQ = 8'h00;
  assign FL_ADDR = 22'h000000;
  assign GPIO_0 = 36'h000000000;
  assign GPIO_1 = 36'h000000000;
  assign DRAM_ADDR = 12'h000;
  assign DRAM_DQ = 16'h0000;
  assign DRAM_LDQM = 1'b0;
  assign DRAM_UDQM = 1'b0;
  assign DRAM_WE_N = 1'b1;
  assign DRAM_CAS_N = 1'b1;
  assign DRAM_RAS_N = 1'b1;
  assign DRAM_CS_N = 1'b1;
  assign DRAM_BA_0 = 1'b0;
  assign DRAM_BA_1 = 1'b0;
  assign DRAM_CLK = 1'b0;
  assign DRAM_CKE = 1'b0;
  assign SD_DAT = 1'b1;
  assign SD_DAT3 = 1'b1;
  assign SD_CMD = 1'b1;
  assign SD_CLK = 1'b0;
  assign I2C_SDAT = 1'b0;
  assign I2C_SCLK = 1'b0;
  assign TDO = 1'b0;
  assign AUD_ADCLRCK = 1'b0;
  assign AUD_DACLRCK = 1'b0;
  assign AUD_DACDAT = 1'b0;
  assign AUD_BCLK = 1'b0;
  assign AUD_XCK = 1'b0;

endmodule
